@@ -1,10 +1,10 @@
-use eva_common::events::{LocalStateEvent, RawStateEventOwned};
+use eva_common::events::{LocalStateEvent, RawStateEventOwned, RemoteStateEvent};
 use eva_common::prelude::*;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use std::fmt;
 use std::str::FromStr;
 
-/// basic state object, collected usually from topics
+/// basic state object, usually collected from topics, ignores extra fields
 #[derive(Serialize, Deserialize, Clone, Debug)]
 pub struct State {
     pub status: ItemStatus,
@@ -13,7 +13,8 @@ pub struct State {
     pub set_time: f64,
 }
 
-/// basic state with OID specified inside the object
+/// basic state with OID specified inside the object, collected from RPC, useful to keep/transfer
+/// via internal channels
 #[derive(Serialize, Deserialize, Clone, Debug)]
 pub struct ItemState {
     pub oid: OID,
@@ -46,7 +47,8 @@ impl ItemState {
     }
 }
 
-/// short item state, without set-time
+/// short item state, without set-time, collected from RPC, useful to keep/transfer via internal
+/// channels
 #[derive(Serialize, Deserialize, Clone, Debug)]
 pub struct ShortItemState {
     pub oid: OID,
@@ -68,6 +70,7 @@ impl ShortItemState {
     }
 }
 
+/// helper object for database services - status can be null/absent
 #[derive(Serialize, Deserialize, Clone, Debug)]
 pub struct HistoricalState {
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -78,6 +81,7 @@ pub struct HistoricalState {
     pub set_time: f64,
 }
 
+/// result object for database services
 #[derive(Serialize, Deserialize, Clone, Debug)]
 pub struct CompactStateHistory {
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -291,12 +295,14 @@ impl FromStr for StateProp {
     }
 }
 
+/// Full item state, used by replication services for bulk topics
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct FullItemState {
     pub oid: OID,
     pub status: ItemStatus,
     pub value: Value,
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub act: Option<usize>,
     pub ieid: IEID,
     pub t: f64,
@@ -326,3 +332,45 @@ impl From<FullItemState> for LocalStateEvent {
         }
     }
 }
+
+/// Full item state for remote items, used by HMI services
+#[derive(Debug, Clone, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub struct FullRemoteItemState {
+    pub oid: OID,
+    pub status: ItemStatus,
+    pub value: Value,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub act: Option<usize>,
+    pub ieid: IEID,
+    pub t: f64,
+    pub node: String,
+    pub connected: bool,
+}
+
+impl FullRemoteItemState {
+    pub fn from_local_state_event(state: LocalStateEvent, oid: OID, system_name: &str) -> Self {
+        Self {
+            oid,
+            status: state.status,
+            value: state.value,
+            act: state.act,
+            ieid: state.ieid,
+            t: state.t,
+            node: system_name.to_owned(),
+            connected: true,
+        }
+    }
+    pub fn from_remote_state_event(state: RemoteStateEvent, oid: OID) -> Self {
+        Self {
+            oid,
+            status: state.status,
+            value: state.value,
+            act: state.act,
+            ieid: state.ieid,
+            t: state.t,
+            node: state.node,
+            connected: state.connected,
+        }
+    }
+}
